@@ -134,12 +134,11 @@ def load_conf(path=None, file=None):
         try:
             confpath = os.environ["DULWICH_SWIFT_CFG"]
         except KeyError as exc:
-            raise Exception(
-                "You need to specify a configuration file") from exc
+            raise Exception("You need to specify a configuration file") from exc
     else:
         confpath = path
     if not os.path.isfile(confpath):
-        raise Exception("Unable to read configuration file %s" % confpath)
+        raise Exception(f"Unable to read configuration file {confpath}")
     conf.read(confpath)
     return conf
 
@@ -258,11 +257,7 @@ class SwiftConnector:
         if ret.status_code < 200 or ret.status_code >= 300:
             raise SwiftException(
                 "AUTH v1.0 request failed on "
-                + "{} with error code {} ({})".format(
-                    str(auth_httpclient.get_base_url()) + path,
-                    ret.status_code,
-                    str(ret.items()),
-                )
+                + f"{self.auth_url} with error code {ret.status_code} ({ret.items()!s})"
             )
         storage_url = ret["X-Storage-Url"]
         token = ret["X-Auth-Token"]
@@ -293,20 +288,18 @@ class SwiftConnector:
         if ret.status_code < 200 or ret.status_code >= 300:
             raise SwiftException(
                 "AUTH v2.0 request failed on "
-                + "{} with error code {} ({})".format(
-                    str(auth_httpclient.get_base_url()) + path,
-                    ret.status_code,
-                    str(ret.items()),
-                )
+                + f"{str(auth_httpclient.get_base_url()) + path} with error code {ret.status_code} ({ret.items()!s})"
             )
         auth_ret_json = json.loads(ret.read())
         token = auth_ret_json["access"]["token"]["id"]
         catalogs = auth_ret_json["access"]["serviceCatalog"]
-        object_store = [
+        object_store = next(
             o_store for o_store in catalogs if o_store["type"] == "object-store"
-        ][0]
+        )
         endpoints = object_store["endpoints"]
-        endpoint = [endp for endp in endpoints if endp["region"] == self.region_name][0]
+        endpoint = next(
+            endp for endp in endpoints if endp["region"] == self.region_name
+        )
         return endpoint[self.endpoint_type], token
 
     def test_root_exists(self):
@@ -319,7 +312,7 @@ class SwiftConnector:
             return None
         if ret.status_code < 200 or ret.status_code > 300:
             raise SwiftException(
-                "HEAD request failed with error code %s" % ret.status_code
+                f"HEAD request failed with error code {ret.status_code}"
             )
         return True
 
@@ -333,7 +326,7 @@ class SwiftConnector:
             ret = self.httpclient.request("PUT", self.base_path)
             if ret.status_code < 200 or ret.status_code > 300:
                 raise SwiftException(
-                    "PUT request failed with error code %s" % ret.status_code
+                    f"PUT request failed with error code {ret.status_code}"
                 )
 
     def get_container_objects(self):
@@ -349,7 +342,7 @@ class SwiftConnector:
             return None
         if ret.status_code < 200 or ret.status_code > 300:
             raise SwiftException(
-                "GET request failed with error code %s" % ret.status_code
+                f"GET request failed with error code {ret.status_code}"
             )
         content = ret.read()
         return json.loads(content)
@@ -368,7 +361,7 @@ class SwiftConnector:
             return None
         if ret.status_code < 200 or ret.status_code > 300:
             raise SwiftException(
-                "HEAD request failed with error code %s" % ret.status_code
+                f"HEAD request failed with error code {ret.status_code}"
             )
         resp_headers = {}
         for header, value in ret.items():
@@ -402,7 +395,7 @@ class SwiftConnector:
 
         if ret.status_code < 200 or ret.status_code > 300:
             raise SwiftException(
-                "PUT request failed with error code %s" % ret.status_code
+                f"PUT request failed with error code {ret.status_code}"
             )
 
     def get_object(self, name, range=None):
@@ -417,14 +410,14 @@ class SwiftConnector:
         """
         headers = {}
         if range:
-            headers["Range"] = "bytes=%s" % range
+            headers["Range"] = f"bytes={range}"
         path = self.base_path + "/" + name
         ret = self.httpclient.request("GET", path, headers=headers)
         if ret.status_code == 404:
             return None
         if ret.status_code < 200 or ret.status_code > 300:
             raise SwiftException(
-                "GET request failed with error code %s" % ret.status_code
+                f"GET request failed with error code {ret.status_code}"
             )
         content = ret.read()
 
@@ -444,7 +437,7 @@ class SwiftConnector:
         ret = self.httpclient.request("DELETE", path)
         if ret.status_code < 200 or ret.status_code > 300:
             raise SwiftException(
-                "DELETE request failed with error code %s" % ret.status_code
+                f"DELETE request failed with error code {ret.status_code}"
             )
 
     def del_root(self):
@@ -458,7 +451,7 @@ class SwiftConnector:
         ret = self.httpclient.request("DELETE", self.base_path)
         if ret.status_code < 200 or ret.status_code > 300:
             raise SwiftException(
-                "DELETE request failed with error code %s" % ret.status_code
+                f"DELETE request failed with error code {ret.status_code}"
             )
 
 
@@ -687,7 +680,7 @@ class SwiftObjectStore(PackBasedObjectStore):
             if entries:
                 basename = posixpath.join(
                     self.pack_dir,
-                    "pack-%s" % iter_sha1(entry[0] for entry in entries),
+                    f"pack-{iter_sha1(entry[0] for entry in entries)}",
                 )
                 index = BytesIO()
                 write_pack_index_v2(index, entries, pack.get_stored_checksum())
@@ -875,10 +868,10 @@ class SwiftRepo(BaseRepo):
         self.scon = SwiftConnector(self.root, self.conf)
         objects = self.scon.get_container_objects()
         if not objects:
-            raise Exception("There is not any GIT repo here : %s" % self.root)
+            raise Exception(f"There is not any GIT repo here : {self.root}")
         objects = [o["name"].split("/")[0] for o in objects]
         if OBJECTDIR not in objects:
-            raise Exception("This repository (%s) is not bare." % self.root)
+            raise Exception(f"This repository ({self.root}) is not bare.")
         self.bare = True
         self._controldir = self.root
         object_store = SwiftObjectStore(self.scon)
@@ -1014,12 +1007,14 @@ def main(argv=sys.argv):
     }
 
     if len(sys.argv) < 2:
-        print("Usage: {} <{}> [OPTIONS...]".format(sys.argv[0], "|".join(commands.keys())))
+        print(
+            "Usage: {} <{}> [OPTIONS...]".format(sys.argv[0], "|".join(commands.keys()))
+        )
         sys.exit(1)
 
     cmd = sys.argv[1]
     if cmd not in commands:
-        print("No such subcommand: %s" % cmd)
+        print(f"No such subcommand: {cmd}")
         sys.exit(1)
     commands[cmd](sys.argv[2:])
 
